@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: 09799da0e2077050c2b581d8c750723677c907e0 $
+ * Version:	$Id: 9dca00733651fd11277b685c0938e07cc0049f4c $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2002  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 09799da0e2077050c2b581d8c750723677c907e0 $")
+RCSID("$Id: 9dca00733651fd11277b685c0938e07cc0049f4c $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -509,16 +509,6 @@ static ssize_t xlat_listen_common(REQUEST *request, rad_listen_t *listen,
 	}
 #endif
 
-#ifdef WITH_COA_TUNNEL
-	/*
-	 *      Look for RADSEC CoA tunnel key.
-	 */
-	if (listen->key && (strcmp(fmt, "Originating-Realm-Key") == 0)) {
-		strlcpy(out, listen->key, outlen);
-		return strlen(out);
-	}
-#endif
-
 	cp = cf_pair_find(listen->cs, fmt);
 	if (!cp || !(value = cf_pair_value(cp))) {
 		RDEBUG("Listener does not contain config item \"%s\"", fmt);
@@ -572,7 +562,6 @@ static int switch_users(CONF_SECTION *cs)
 	 *	initialized.
 	 */
 	if (fr_set_dumpable_init() < 0) {
-		fr_perror("%s", main_config.name);
 		return 0;
 	}
 
@@ -583,8 +572,7 @@ static int switch_users(CONF_SECTION *cs)
 	if (rad_debug_lvl && (getuid() != 0)) return 1;
 
 	if (cf_section_parse(cs, NULL, bootstrap_config) < 0) {
-		fr_strerror_printf("%s: Error: Failed to parse user/group information.\n",
-				   main_config.name);
+		fr_strerror_printf("Failed to parse user/group information.");
 		return 0;
 	}
 
@@ -599,8 +587,8 @@ static int switch_users(CONF_SECTION *cs)
 
 		gr = getgrnam(gid_name);
 		if (!gr) {
-			fr_strerror_printf("%s: Cannot get ID for group %s: %s\n",
-					   main_config.name, gid_name, fr_syserror(errno));
+			fr_strerror_printf("Cannot get ID for group %s: %s",
+					   gid_name, fr_syserror(errno));
 			return 0;
 		}
 
@@ -620,8 +608,8 @@ static int switch_users(CONF_SECTION *cs)
 		struct passwd *user;
 
 		if (rad_getpwnam(cs, &user, uid_name) < 0) {
-			fr_strerror_printf("%s: Cannot get passwd entry for user %s: %s\n",
-					   main_config.name, uid_name, fr_strerror());
+			fr_strerror_printf("Cannot get passwd entry for user %s: %s",
+					   uid_name, fr_strerror());
 			return 0;
 		}
 
@@ -633,8 +621,8 @@ static int switch_users(CONF_SECTION *cs)
 			do_suid = true;
 #ifdef HAVE_INITGROUPS
 			if (initgroups(uid_name, server_gid) < 0) {
-				fr_strerror_printf("%s: Cannot initialize supplementary group list for user %s: %s\n",
-						   main_config.name, uid_name, fr_syserror(errno));
+				fr_strerror_printf("Cannot initialize supplementary group list for user %s: %s",
+						   uid_name, fr_syserror(errno));
 				talloc_free(user);
 				return 0;
 			}
@@ -649,8 +637,8 @@ static int switch_users(CONF_SECTION *cs)
 	 */
 	if (chroot_dir) {
 		if (chroot(chroot_dir) < 0) {
-			fr_strerror_printf("%s: Failed to perform chroot %s: %s",
-					   main_config.name, chroot_dir, fr_syserror(errno));
+			fr_strerror_printf("Failed to perform chroot to %s: %s",
+					   chroot_dir, fr_syserror(errno));
 			return 0;
 		}
 
@@ -677,8 +665,8 @@ static int switch_users(CONF_SECTION *cs)
 	 */
 	if (do_sgid) {
 		if (setgid(server_gid) < 0){
-			fr_strerror_printf("%s: Failed setting group to %s: %s",
-					   main_config.name, gid_name, fr_syserror(errno));
+			fr_strerror_printf("Failed setting group to %s: %s",
+					   gid_name, fr_syserror(errno));
 			return 0;
 		}
 	}
@@ -725,8 +713,8 @@ static int switch_users(CONF_SECTION *cs)
 		default_log.fd = open(main_config.log_file,
 				      O_WRONLY | O_APPEND | O_CREAT, 0640);
 		if (default_log.fd < 0) {
-			fr_strerror_printf("%s: Failed to open log file %s: %s\n",
-					   main_config.name, main_config.log_file, fr_syserror(errno));
+			fr_strerror_printf("Failed to open log file %s: %s\n",
+					   main_config.log_file, fr_syserror(errno));
 			return 0;
 		}
 	}
@@ -742,8 +730,8 @@ static int switch_users(CONF_SECTION *cs)
 	if ((do_suid || do_sgid) &&
 	    (default_log.dst == L_DST_FILES)) {
 		if (fchown(default_log.fd, server_uid, server_gid) < 0) {
-			fr_strerror_printf("%s: Cannot change ownership of log file %s: %s\n",
-					   main_config.name, main_config.log_file, fr_syserror(errno));
+			fr_strerror_printf("Cannot change ownership of log file %s: %s\n",
+					   main_config.log_file, fr_syserror(errno));
 			return 0;
 		}
 	}
@@ -762,7 +750,7 @@ static int switch_users(CONF_SECTION *cs)
 	 *	aren't allowed.
 	 */
 	if (fr_set_dumpable(allow_core_dumps) < 0) {
-		ERROR("%s", fr_strerror());
+		WARN("Failed to allow core dumps - %s", fr_strerror());
 	}
 
 	if (allow_core_dumps) {
@@ -1081,7 +1069,10 @@ do {\
 	/*
 	 *	Switch users as early as possible.
 	 */
-	if (!switch_users(cs)) fr_exit(1);
+	if (!switch_users(cs)) {
+		fprintf(stderr, "%s: ERROR - %s\n", main_config.name, fr_strerror());
+		fr_exit(1);
+	}
 #endif
 
 	/*
