@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: ad5f546639487f356573f50fbf9a74ff9443b6fb $
+ * Version:     $Id: 652c3a8785c459a4404c72cd4d9a7b83237bb1f9 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2007  Alan DeKok <aland@deployingradius.com>
  */
 
-RCSID("$Id: ad5f546639487f356573f50fbf9a74ff9443b6fb $")
+RCSID("$Id: 652c3a8785c459a4404c72cd4d9a7b83237bb1f9 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/realms.h>
@@ -73,10 +73,6 @@ static const FR_NAME_NUMBER home_server_types[] = {
 	{ "acct",		HOME_TYPE_ACCT },
 	{ "auth+acct",		HOME_TYPE_AUTH_ACCT },
 	{ "coa",		HOME_TYPE_COA },
-#ifdef WITH_COA_TUNNEL
-	{ "auth+coa",		HOME_TYPE_AUTH_COA },
-	{ "auth+acct+coa",	HOME_TYPE_AUTH_ACCT_COA },
-#endif
 	{ NULL, 0 }
 };
 
@@ -100,7 +96,7 @@ static realm_config_t *realm_config = NULL;
 static rbtree_t	*home_servers_byaddr = NULL;
 static rbtree_t	*home_servers_byname = NULL;
 #ifdef WITH_STATS
-static int home_server_max_number = 0;
+int home_server_max_number = 0;
 static rbtree_t	*home_servers_bynumber = NULL;
 #endif
 
@@ -437,16 +433,6 @@ static CONF_PARSER home_server_coa[] = {
 	{ "mrd",  FR_CONF_OFFSET(PW_TYPE_INTEGER, home_server_t, coa_mrd), STRINGIFY(30) },
 	CONF_PARSER_TERMINATOR
 };
-
-
-
-#ifdef WITH_COA_TUNNEL
-static CONF_PARSER home_server_recv_coa[] = {
-	{ "virtual_server",  FR_CONF_OFFSET(PW_TYPE_STRING, home_server_t, recv_coa_server), NULL },
-	CONF_PARSER_TERMINATOR
-};
-#endif
-
 #endif
 
 static CONF_PARSER home_server_config[] = {
@@ -496,9 +482,6 @@ static CONF_PARSER home_server_config[] = {
 
 #ifdef WITH_COA
 	{ "coa", FR_CONF_POINTER(PW_TYPE_SUBSECTION, NULL), (void const *) home_server_coa },
-#ifdef WITH_COA_TUNNEL
-	{ "recv_coa", FR_CONF_POINTER(PW_TYPE_SUBSECTION, NULL), (void const *) home_server_recv_coa },
-#endif
 #endif
 
 	CONF_PARSER_TERMINATOR
@@ -698,20 +681,6 @@ bool realm_home_server_add(home_server_t *home)
 		}
 	}
 
-#ifdef WITH_COA_TUNNEL
-	if (home->recv_coa) {
-		if (!home->tls) {
-			ERROR("TLS is required in order to accept CoA requests from a home server");
-			return false;
-		}
-
-		if (!home->recv_coa_server) {
-			ERROR("A 'virtual_server' configuration is required in order to accept CoA requests from a home server");
-			return false;
-		}
-	}
-#endif
-
 	/*
 	 *	Mark it as already processed
 	 */
@@ -824,17 +793,6 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 				goto error;
 			}
 			break;
-
-#ifdef WITH_COA_TUNNEL
-		case HOME_TYPE_AUTH_ACCT_COA:
-			home->dual = true;
-			home->recv_coa = true;
-			break;
-
-		case HOME_TYPE_AUTH_COA:
-			home->recv_coa = true;
-			break;
-#endif
 #endif
 
   		case HOME_TYPE_INVALID:
@@ -861,10 +819,6 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 			}
 
 			if (((home->type == HOME_TYPE_AUTH) ||
-#ifdef WITH_COA_TUNNEL
-			     (home->type == HOME_TYPE_AUTH_COA) ||
-			     (home->type == HOME_TYPE_AUTH_ACCT_COA) ||
-#endif
 			     (home->type == HOME_TYPE_AUTH_ACCT)) && !home->ping_user_password) {
 				cf_log_err_cs(cs, "You must supply a 'password' to enable status_check=request");
 				goto error;
@@ -930,38 +884,6 @@ home_server_t *home_server_afrom_cs(TALLOC_CTX *ctx, realm_config_t *rc, CONF_SE
 	}
 #endif
 
-	/*
-	 *	Check the reverse CoA configuration.
-	 */
-#ifdef WITH_COA_TUNNEL
-	if (home->recv_coa) {
-		if (!tls) {
-			ERROR("TLS is required in order to accept CoA requests from a home server");
-			goto error;
-		}
-
-		if (!home->recv_coa_server) {
-			ERROR("A 'virtual_server' configuration is required in order to accept CoA requests from a home server");
-			goto error;
-		}
-
-		/*
-		 *	Try and find a 'server' section off the root of
-		 *	the config with a name that matches the coa
-		 *	virtual_server.
-		 */
-		if (!rc) {
-			ERROR("Dynamic home servers cannot accept CoA requests");
-			goto error;
-		}
-
-		if (!cf_section_sub_find_name2(rc->cs, "server", home->recv_coa_server)) {
-			cf_log_err_cs(cs, "No such coa server %s", home->recv_coa_server);
-			goto error;
-		}
-	}
-#endif
-
 	/*
 	 *	If were doing RADSEC (tls+tcp) the secret should default
 	 *	to radsec, else a secret must be set.
@@ -1206,16 +1128,6 @@ static int pool_check_home_server(UNUSED realm_config_t *rc, CONF_PAIR *cp,
 	case HOME_TYPE_ACCT:
 		myhome.type = HOME_TYPE_AUTH_ACCT;
 		home = rbtree_finddata(home_servers_byname, &myhome);
-#ifdef WITH_COA_TUNNEL
-		if (!home) {
-			myhome.type = HOME_TYPE_AUTH_COA;
-			home = rbtree_finddata(home_servers_byname, &myhome);
-			if(!home) {
-				myhome.type = HOME_TYPE_AUTH_ACCT_COA;
-				home = rbtree_finddata(home_servers_byname, &myhome);
-			}
-		}
-#endif
 		if (home) {
 			*phome = home;
 			return 1;
@@ -1507,16 +1419,6 @@ static int server_pool_add(realm_config_t *rc,
 			case HOME_TYPE_ACCT:
 				myhome.type = HOME_TYPE_AUTH_ACCT;
 				home = rbtree_finddata(home_servers_byname, &myhome);
-#ifdef WITH_COA_TUNNEL
-				if (!home) {
-					myhome.type = HOME_TYPE_AUTH_COA;
-					home = rbtree_finddata(home_servers_byname, &myhome);
-					if (!home) {
-						myhome.type = HOME_TYPE_AUTH_ACCT_COA;
-						home = rbtree_finddata(home_servers_byname, &myhome);
-					}
-				}
-#endif
 				break;
 
 			default:
@@ -3080,19 +2982,17 @@ int home_server_afrom_file(char const *filename)
 
 	home->dynamic = true;
 
-	if (home->virtual_server || home->dual) {
-		fr_strerror_printf("Dynamic home_server '%s' cannot have 'server' or 'auth+acct'", p);
+	if (home->virtual_server) {
+		fr_strerror_printf("Dynamic home_server '%s' cannot have 'server = %s' configuration item", p, home->virtual_server);
 		talloc_free(home);
 		goto error;
 	}
 
-#ifdef COA_TUNNEL
-	if (home->recv_coa) {
-		fr_strerror_printf("Dynamic home_server '%s' cannot receive CoA requests'", p);
+	if (home->dual) {
+		fr_strerror_printf("Dynamic home_server '%s' is missing 'type', or it is set to 'auth+acct'.  Please specify 'type = auth' or 'type = acct', etc.", p);
 		talloc_free(home);
 		goto error;
 	}
-#endif
 
 	if (!realm_home_server_add(home)) {
 		fr_strerror_printf("Failed adding home_server to the internal data structures");
