@@ -1,7 +1,7 @@
 /*
  * tls.c
  *
- * Version:     $Id: 66040e6980bdcf2f933471190a47841ab657d819 $
+ * Version:     $Id: c3c40d17ea28ea31a3a86ae5ee0a2aa68f903f7a $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 66040e6980bdcf2f933471190a47841ab657d819 $")
+RCSID("$Id: c3c40d17ea28ea31a3a86ae5ee0a2aa68f903f7a $")
 USES_APPLE_DEPRECATED_API	/* OpenSSL API has been deprecated by Apple */
 
 #include <freeradius-devel/radiusd.h>
@@ -118,216 +118,9 @@ static int CC_HINT(nonnull) tls_socket_write(rad_listen_t *listener, REQUEST *re
 
 	return 1;
 }
-
-/*
- *	Check for PROXY protocol.  Once that's done, clear
- *	listener->proxy_protocol.
- */
-static int proxy_protocol_check(rad_listen_t *listener, REQUEST *request)
-{
-	listen_socket_t *sock = listener->data;
-	uint8_t const *p, *end, *eol;
-	int af, argc, src_port, dst_port;
-	unsigned long num;
-	fr_ipaddr_t src, dst;
-	char *argv[5], *eos;
-	ssize_t rcode;
-	RADCLIENT *client;
-
-	/*
-	 *	Begin by trying to fill the buffer.
-	 */
-	rcode = read(request->packet->sockfd,
-		     sock->ssn->dirty_in.data + sock->ssn->dirty_in.used,
-		     sizeof(sock->ssn->dirty_in.data) - sock->ssn->dirty_in.used);
-	if (rcode < 0) {
-		if (errno == EINTR) return 0;
-		RDEBUG("(TLS) Closing PROXY socket from client port %u due to read error - %s", sock->other_port, fr_syserror(errno));
-		return -1;
-	}
-
-	if (rcode == 0) {
-		DEBUG("(TLS) Closing PROXY socket from client port %u - other end closed connection", sock->other_port);
-		return -1;
-	}
-
-	/*
-	 *	We've read data, scan the buffer for a CRLF.
-	 */
-	sock->ssn->dirty_in.used += rcode;
-
-	dump_hex("READ FROM PROXY PROTOCOL SOCKET", sock->ssn->dirty_in.data, sock->ssn->dirty_in.used);
-
-	p = sock->ssn->dirty_in.data;
-
-	/*
-	 *	CRLF MUST be within the first 107 bytes.
-	 */
-	if (sock->ssn->dirty_in.used < 107) {
-		end = p + sock->ssn->dirty_in.used;
-	} else {
-		end = p + 107;
-	}
-	eol = NULL;
-
-	/*
-	 *	Scan for CRLF.
-	 */
-	while ((p + 1) < end) {
-		if ((p[0] == 0x0d) && (p[1] == 0x0a)) {
-			eol = p;
-			break;
-		}
-
-		/*
-		 *	Other control characters, or non-ASCII data.
-		 *	That's a problem.
-		 */
-		if ((*p < ' ') || (*p >= 0x80)) {
-		invalid_data:
-			DEBUG("(TLS) Closing PROXY socket from client port %u - received invalid data", sock->other_port);
-			return -1;
-		}
-
-		p++;
-	}
-
-	/*
-	 *	No CRLF, keep reading until we have it.
-	 */
-	if (!eol) return 0;
-
-	p = sock->ssn->dirty_in.data;
-
-	/*
-	 *	Let's see if the PROXY line is well-formed.
-	 */
-	if ((eol - p) < 14) goto invalid_data;
-	
-	/*
-	 *	We only support TCP4 and TCP6.
-	 */
-	if (memcmp(p, "PROXY TCP", 9) != 0) goto invalid_data;
-
-	p += 9;
-
-	if (*p == '4') {
-		af = AF_INET;
-
-	} else if (*p == '6') {
-		af = AF_INET6;
-
-	} else goto invalid_data;
-
-	p++;
-	if (*p != ' ') goto invalid_data;
-	p++;
-
-	sock->ssn->dirty_in.data[eol - sock->ssn->dirty_in.data] = '\0'; /* overwite the CRLF */
-
-	/*
-	 *	Parse the fields (being a little forgiving), while
-	 *	checking for too many / too few fields.
-	 */
-	argc = str2argv((char *) &sock->ssn->dirty_in.data[p - sock->ssn->dirty_in.data], (char **) &argv, 5);
-	if (argc != 4) goto invalid_data;
-
-	memset(&src, 0, sizeof(src));
-	memset(&dst, 0, sizeof(dst));
-
-	if (fr_pton(&src, argv[0], -1, af, false) < 0) goto invalid_data;
-	if (fr_pton(&dst, argv[1], -1, af, false) < 0) goto invalid_data;
-
-	num = strtoul(argv[2], &eos, 10);
-	if (num > 65535) goto invalid_data;
-	if (*eos) goto invalid_data;
-	src_port = num;
-
-	num = strtoul(argv[3], &eos, 10);
-	if (num > 65535) goto invalid_data;
-	if (*eos) goto invalid_data;
-	dst_port = num;
-
-	/*
-	 *	And copy the various fields around.
-	 */
-	sock->haproxy_src_ipaddr = sock->other_ipaddr;
-	sock->haproxy_src_port = sock->other_port;
-
-	sock->haproxy_dst_ipaddr = sock->my_ipaddr;
-	sock->haproxy_dst_port = sock->my_port;
-
-	sock->my_ipaddr = dst;
-	sock->my_port = dst_port;
-
-	sock->other_ipaddr = src;
-	sock->other_port = src_port;
-
-	/*
-	 *	Print out what we've changed.  Note that the TCP
-	 *	socket address family and the PROXY address family may
-	 *	be different!
-	 */
-	if (RDEBUG_ENABLED) {
-		char src_buf[128], dst_buf[128];
-
-		RDEBUG("(TLS) Received PROXY protocol connection from client %s:%s -> %s:%s, via proxy %s:%u -> %s:%u",
-		       argv[0], argv[2], argv[1], argv[3],
-		       inet_ntop(af, &sock->haproxy_src_ipaddr.ipaddr, src_buf, sizeof(src_buf)),
-		       sock->haproxy_src_port,
-		       inet_ntop(af, &sock->haproxy_dst_ipaddr.ipaddr, dst_buf, sizeof(dst_buf)),
-		       sock->haproxy_dst_port);
-	}
-
-        /*
-         *      Ensure that the source IP indicated by the PROXY
-         *      protocol is a known TLS client.
-         */
-        if ((client = client_listener_find(listener, &src, src_port)) == NULL ||
-             client->proto != IPPROTO_TCP) {
-		RDEBUG("(TLS) Unknown client %s - dropping PROXY protocol connection", argv[0]);
-		return -1;
-        }
-
-        /*
-         *      Use the client indicated by the proxy.
-         */
-        sock->client = client;
-
-	/*
-         *      Fix up the current request so that the first packet's
-         *      src/dst is valid.  Subsequent packets will get the
-         *      clients IP from the listener and listen_sock
-         *      structures.
-         */
-        request->packet->dst_ipaddr = dst;
-        request->packet->dst_port = dst_port;
-        request->packet->src_ipaddr = src;
-        request->packet->src_port = src_port;
-
-	/*
-	 *	Move any remaining TLS data to the start of the buffer.
-	 */
-	eol += 2;
-	end = sock->ssn->dirty_in.data + sock->ssn->dirty_in.used;
-	if (eol < end) {
-		memmove(sock->ssn->dirty_in.data, eol, end - eol);
-		sock->ssn->dirty_in.used = end - eol;
-	} else {
-		sock->ssn->dirty_in.used = 0;
-	}
-		
-	/*
-	 *	It's no longer a PROXY protocol, but just straight TLS.
-	 */
-	listener->proxy_protocol = false;
-
-	return 1;
-}
-
 static int tls_socket_recv(rad_listen_t *listener)
 {
-	bool doing_init = false, already_read = false;
+	bool doing_init = false;
 	ssize_t rcode;
 	RADIUS_PACKET *packet;
 	REQUEST *request;
@@ -392,30 +185,6 @@ static int tls_socket_recv(rad_listen_t *listener)
 
 	request = sock->request;
 
-	/*
-	 *	Bypass ALL of the TLS stuff until we've read the PROXY
-	 *	header.
-	 *
-	 *	If the PROXY header checks pass, then the flag is
-	 *	cleared, as we don't need it any more.
-	 */
-	if (listener->proxy_protocol) {
-		rcode = proxy_protocol_check(listener, request);
-		if (rcode < 0) {
-			RDEBUG("(TLS) Closing PROXY TLS socket from client port %u", sock->other_port);
-			tls_socket_close(listener);
-			return 0;
-		}
-		if (rcode == 0) return 1;
-
-		/*
-		 *	The buffer might already have data.  In that
-		 *	case, we don't want to do a blocking read
-		 *	later.
-		 */
-		already_read = (sock->ssn->dirty_in.used > 0);
-	}
-
 	if (sock->state == LISTEN_TLS_SETUP) {
 		RDEBUG3("(TLS) Setting connection state to RUNNING");
 		sock->state = LISTEN_TLS_RUNNING;
@@ -441,31 +210,28 @@ static int tls_socket_recv(rad_listen_t *listener)
 		goto check_for_setup;
 	}
 
-	if (!already_read) {
-		rcode = read(request->packet->sockfd,
-			     sock->ssn->dirty_in.data,
-			     sizeof(sock->ssn->dirty_in.data));
-		if ((rcode < 0) && (errno == ECONNRESET)) {
-		do_close:
-			DEBUG("(TLS) Closing socket from client port %u", sock->other_port);
-			tls_socket_close(listener);
-			PTHREAD_MUTEX_UNLOCK(&sock->mutex);
-			return 0;
-		}
-
-		if (rcode < 0) {
-			RDEBUG("(TLS) Error reading socket: %s", fr_syserror(errno));
-			goto do_close;
-		}
-
-		/*
-		 *	Normal socket close.
-		 */
-		if (rcode == 0) goto do_close;
-
-		sock->ssn->dirty_in.used = rcode;
+	rcode = read(request->packet->sockfd,
+		     sock->ssn->dirty_in.data,
+		     sizeof(sock->ssn->dirty_in.data));
+	if ((rcode < 0) && (errno == ECONNRESET)) {
+	do_close:
+		DEBUG("(TLS) Closing socket from client port %u", sock->other_port);
+		tls_socket_close(listener);
+		PTHREAD_MUTEX_UNLOCK(&sock->mutex);
+		return 0;
 	}
 
+	if (rcode < 0) {
+		RDEBUG("(TLS) Error reading socket: %s", fr_syserror(errno));
+		goto do_close;
+	}
+
+	/*
+	 *	Normal socket close.
+	 */
+	if (rcode == 0) goto do_close;
+
+	sock->ssn->dirty_in.used = rcode;
 	dump_hex("READ FROM SSL", sock->ssn->dirty_in.data, sock->ssn->dirty_in.used);
 
 	/*
@@ -496,8 +262,14 @@ check_for_setup:
 		}
 
 		/*
-		 *	Else we MUST be finished the SSL setup.
+		 *      If SSL handshake still isn't finished, then there
+		 *      is more data to read.  Release the mutex and
+		 *      return so this function will be called again
 		 */
+		if (!SSL_is_init_finished(sock->ssn->ssl)) {
+			PTHREAD_MUTEX_UNLOCK(&sock->mutex);
+			return 0;
+		}
 	}
 
 	/*
@@ -639,9 +411,6 @@ int dual_tls_recv(rad_listen_t *listener)
 	listen_socket_t *sock = listener->data;
 	RADCLIENT	*client = sock->client;
 	BIO		*rbio;
-#ifdef WITH_COA_TUNNEL
-	bool		is_reply = false;
-#endif
 
 	if (listener->status != RAD_LISTEN_STATUS_KNOWN) return 0;
 
@@ -700,14 +469,6 @@ redo:
 		FR_STATS_INC(dsc, total_requests);
 		fun = rad_coa_recv;
 		break;
-
-#ifdef WITH_COA_TUNNEL
-	case PW_CODE_COA_ACK:
-	case PW_CODE_COA_NAK:
-		if (!listener->send_coa) goto bad_packet;
-		is_reply = true;
-                break;
-#endif
 #endif
 
 	case PW_CODE_STATUS_SERVER:
@@ -734,15 +495,6 @@ redo:
 		return 0;
 	} /* switch over packet types */
 
-#ifdef WITH_COA_TUNNEL
-	if (is_reply) {
-		if (!request_proxy_reply(packet)) {
-			rad_free(&packet);
-			return 0;
-		}
-	} else
-#endif
-
 	if (!request_receive(NULL, listener, packet, client, fun)) {
 		FR_STATS_INC(auth, total_packets_dropped);
 		rad_free(&packet);
@@ -823,21 +575,6 @@ int dual_tls_send(rad_listen_t *listener, REQUEST *request)
 	 */
 	if (request->reply->code == 0) return 0;
 
-#ifdef WITH_COA_TUNNEL
-	/*
-	 *	Save the key, if we haven't already done that.
-	 */
-	if (listener->send_coa && !listener->key) {
-		VALUE_PAIR *vp = NULL;
-
-		vp = fr_pair_find_by_num(request->config, PW_ORIGINATING_REALM_KEY, 0, TAG_ANY);
-		if (vp) {
-			RDEBUG("Adding send CoA listener with key %s", vp->vp_strvalue);
-			listen_coa_add(request->listener, vp->vp_strvalue);
-		}
-	}
-#endif
-
 	/*
 	 *	Pack the VPs
 	 */
@@ -889,58 +626,6 @@ int dual_tls_send(rad_listen_t *listener, REQUEST *request)
 	return 0;
 }
 
-#ifdef WITH_COA_TUNNEL
-/*
- *	Send a CoA request to a NAS, as a proxied packet.
- *
- *	The proxied packet MUST already have been encoded.
- */
-int dual_tls_send_coa_request(rad_listen_t *listener, REQUEST *request)
-{
-	listen_socket_t *sock = listener->data;
-
-	VERIFY_REQUEST(request);
-
-	rad_assert(listener->proxy_send == dual_tls_send_coa_request);
-
-	if (listener->status != RAD_LISTEN_STATUS_KNOWN) return 0;
-
-	rad_assert(request->proxy->data);
-
-	if (request->proxy->data_len > (MAX_PACKET_LEN - 100)) {
-		RWARN("Packet is large, and possibly truncated - %zd vs max %d",
-		      request->proxy->data_len, MAX_PACKET_LEN);
-	}
-
-	PTHREAD_MUTEX_LOCK(&sock->mutex);
-
-	/*
-	 *	Write the packet to the SSL buffers.
-	 */
-	sock->ssn->record_plus(&sock->ssn->clean_in,
-			       request->proxy->data, request->proxy->data_len);
-
-	dump_hex("TUNNELED DATA < ", sock->ssn->clean_in.data, sock->ssn->clean_in.used);
-
-	/*
-	 *	Do SSL magic to get encrypted data.
-	 */
-	tls_handshake_send(request, sock->ssn);
-
-	/*
-	 *	And finally write the data to the socket.
-	 */
-	if (sock->ssn->dirty_out.used > 0) {
-		dump_hex("WRITE TO SSL", sock->ssn->dirty_out.data, sock->ssn->dirty_out.used);
-
-		tls_socket_write(listener, request);
-	}
-	PTHREAD_MUTEX_UNLOCK(&sock->mutex);
-
-	return 0;
-}
-#endif
-
 static int try_connect(tls_session_t *ssn)
 {
 	int ret;
@@ -1107,10 +792,6 @@ int proxy_tls_recv(rad_listen_t *listener)
 	RADIUS_PACKET *packet;
 	uint8_t *data;
 	ssize_t data_len;
-#ifdef WITH_COA_TUNNEL
-	bool is_request = false;
-	RADCLIENT *client = sock->client;
-#endif
 
 	if (listener->status != RAD_LISTEN_STATUS_KNOWN) return 0;
 
@@ -1165,25 +846,9 @@ int proxy_tls_recv(rad_listen_t *listener)
 	case PW_CODE_DISCONNECT_NAK:
 		break;
 
-#ifdef WITH_COA_TUNNEL
-	case PW_CODE_COA_REQUEST:
-		if (!listener->send_coa) goto bad_packet;
-		FR_STATS_INC(coa, total_requests);
-		is_request = true;
-		break;
-
-	case PW_CODE_DISCONNECT_REQUEST:
-		if (!listener->send_coa) goto bad_packet;
-		FR_STATS_INC(dsc, total_requests);
-		is_request = true;
-		break;
-#endif
 #endif
 
 	default:
-#ifdef WITH_COA_TUNNEL
-	bad_packet:
-#endif
 		/*
 		 *	FIXME: Update MIB for packet types?
 		 */
@@ -1196,15 +861,6 @@ int proxy_tls_recv(rad_listen_t *listener)
 		return 0;
 	}
 
-#ifdef WITH_COA_TUNNEL
-	if (is_request) {
-		if (!request_receive(NULL, listener, packet, client, rad_coa_recv)) {
-			FR_STATS_INC(auth, total_packets_dropped);
-			rad_free(&packet);
-			return 0;
-		}
-	} else
-#endif
 	if (!request_proxy_reply(packet)) {
 		rad_free(&packet);
 		return 0;
@@ -1275,71 +931,6 @@ int proxy_tls_send(rad_listen_t *listener, REQUEST *request)
 
 	return 1;
 }
-
-#ifdef WITH_COA_TUNNEL
-int proxy_tls_send_reply(rad_listen_t *listener, REQUEST *request)
-{
-	int rcode;
-	listen_socket_t *sock = listener->data;
-
-	VERIFY_REQUEST(request);
-
-	rad_assert(sock->ssn->connected);
-
-	if ((listener->status != RAD_LISTEN_STATUS_INIT &&
-	    (listener->status != RAD_LISTEN_STATUS_KNOWN))) return 0;
-
-	/*
-	 *	Pack the VPs
-	 */
-	if (rad_encode(request->reply, request->packet,
-		       request->client->secret) < 0) {
-		RERROR("Failed encoding packet: %s", fr_strerror());
-		return 0;
-	}
-
-	if (request->reply->data_len > (MAX_PACKET_LEN - 100)) {
-		RWARN("Packet is large, and possibly truncated - %zd vs max %d",
-		      request->reply->data_len, MAX_PACKET_LEN);
-	}
-
-	/*
-	 *	Sign the packet.
-	 */
-	if (rad_sign(request->reply, request->packet,
-		       request->client->secret) < 0) {
-		RERROR("Failed signing packet: %s", fr_strerror());
-		return 0;
-	}
-
-	DEBUG3("Proxy is writing %u bytes to SSL",
-	       (unsigned int) request->reply->data_len);
-	PTHREAD_MUTEX_LOCK(&sock->mutex);
-	rcode = SSL_write(sock->ssn->ssl, request->reply->data,
-			  request->reply->data_len);
-	if (rcode < 0) {
-		int err;
-
-		err = ERR_get_error();
-		switch (err) {
-		case SSL_ERROR_NONE:
-		case SSL_ERROR_WANT_READ:
-		case SSL_ERROR_WANT_WRITE:
-			break;	/* let someone else retry */
-
-		default:
-			tls_error_log(NULL, "Failed in proxy send");
-			DEBUG("Closing TLS socket to home server");
-			tls_socket_close(listener);
-			PTHREAD_MUTEX_UNLOCK(&sock->mutex);
-			return 0;
-		}
-	}
-	PTHREAD_MUTEX_UNLOCK(&sock->mutex);
-
-	return 1;
-}
-#endif	/* WITH_COA_TUNNEL */
 #endif	/* WITH_PROXY */
 
 #endif	/* WITH_TLS */
